package org.firstinspires.ftc.teamcode;

import com.acmerobotics.dashboard.FtcDashboard;
import com.acmerobotics.dashboard.telemetry.MultipleTelemetry;
import com.qualcomm.robotcore.eventloop.opmode.Autonomous;
import com.qualcomm.robotcore.eventloop.opmode.OpMode;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.pedroPathing.localization.Pose;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.BezierCurve;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.BezierLine;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.PathChain;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.Point;

@Autonomous(name = "Blue Non Basket Auto", group = "Competition")
public class BlueNonBasketAuto extends OpMode {
    private Telemetry telemetryA;

    private Follower follower;

    private PathChain path;

    private final Pose startPose = new Pose(8.000, 55.000);

    @Override
    public void init() {
        follower = new Follower(hardwareMap);

        follower.setMaxPower(.6);

        follower.setStartingPose(startPose);

        path = follower.pathBuilder()
                .addPath(
                        // Line 1
                        new BezierCurve(
                                new Point(8.000, 55.000, Point.CARTESIAN),
                                new Point(27.482, 33.750, Point.CARTESIAN),
                                new Point(62.357, 33.107, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 2
                        new BezierLine(
                                new Point(62.357, 33.107, Point.CARTESIAN),
                                new Point(62.000, 27.000, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 3
                        new BezierLine(
                                new Point(62.000, 27.000, Point.CARTESIAN),
                                new Point(10.000, 27.000, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 4
                        new BezierLine(
                                new Point(10.000, 27.000, Point.CARTESIAN),
                                new Point(61.875, 27.000, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 5
                        new BezierLine(
                                new Point(61.875, 27.000, Point.CARTESIAN),
                                new Point(61.714, 17.357, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 6
                        new BezierLine(
                                new Point(61.714, 17.357, Point.CARTESIAN),
                                new Point(14.464, 17.357, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 7
                        new BezierLine(
                                new Point(14.464, 17.357, Point.CARTESIAN),
                                new Point(61.714, 17.357, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 8
                        new BezierLine(
                                new Point(61.714, 17.357, Point.CARTESIAN),
                                new Point(61.554, 8.000, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 9
                        new BezierLine(
                                new Point(61.554, 8.000, Point.CARTESIAN),
                                new Point(12.536, 8.196, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 10
                        new BezierCurve(
                                new Point(12.536, 8.196, Point.CARTESIAN),
                                new Point(52.071, 19.929, Point.CARTESIAN),
                                new Point(50.786, 33.750, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 11
                        new BezierCurve(
                                new Point(50.786, 33.750, Point.CARTESIAN),
                                new Point(2.571, 39.375, Point.CARTESIAN),
                                new Point(20.732, 78.911, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 12
                        new BezierCurve(
                                new Point(20.732, 78.911, Point.CARTESIAN),
                                new Point(24.429, 111.054, Point.CARTESIAN),
                                new Point(46.929, 121.018, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0))
                .addPath(
                        // Line 13
                        new BezierCurve(
                                new Point(46.929, 121.018, Point.CARTESIAN),
                                new Point(68.143, 116.357, Point.CARTESIAN),
                                new Point(63.000, 97.714, Point.CARTESIAN)
                        )
                )
                .setLinearHeadingInterpolation(Math.toRadians(0), Math.toRadians(270)).build();
        follower.followPath(path);

        telemetryA = new MultipleTelemetry(this.telemetry, FtcDashboard.getInstance().getTelemetry());
        telemetryA.update();
    }

    @Override
    public void loop() {
        follower.update();
        follower.telemetryDebug(telemetryA);
    }
}
