package org.firstinspires.ftc.teamcode;

import com.qualcomm.hardware.rev.RevHubOrientationOnRobot;
import com.qualcomm.robotcore.hardware.DcMotorSimple.Direction;

import org.firstinspires.ftc.teamcode.pedroPathing.localization.Encoder;

public class PedroConstants {

    /*
        Motor configuration names
    */
    public static final String FRONT_LEFT_MOTOR = "Drive front lt";
    public static final String BACK_LEFT_MOTOR = "Drive back lt";
    public static final String FRONT_RIGHT_MOTOR = "Drive front rt";
    public static final String BACK_RIGHT_MOTOR = "Drive back rt";

    /*
        Motor directions
     */
    public static final Direction FRONT_LEFT_MOTOR_DIRECTION = Direction.REVERSE;
    public static final Direction BACK_LEFT_MOTOR_DIRECTION = Direction.REVERSE;
    public static final Direction FRONT_RIGHT_MOTOR_DIRECTION = Direction.FORWARD;
    public static final Direction BACK_RIGHT_MOTOR_DIRECTION = Direction.FORWARD;

    /*
        Centricity : true is robot-centric movement; false if field-centric movement
     */
    public static final boolean CENTRICITY = true;

    /*
        Motor Max Power
     */
    public static final double MAX_POWER = .50;

    /*
        IMU
     */
    public static final String IMU = "imu";
    public static final RevHubOrientationOnRobot.LogoFacingDirection IMU_LOGO_FACING_DIRECTION
            = RevHubOrientationOnRobot.LogoFacingDirection.DOWN;
    public static final RevHubOrientationOnRobot.UsbFacingDirection IMU_USB_FACING_DIRECTION
            = RevHubOrientationOnRobot.UsbFacingDirection.LEFT;

    /*
        Dead wheels
     */
    public static final String LEFT_ENCODER = "encoder left";
    public static final String RIGHT_ENCODER = "encoder right";
    public static final String BACK_ENCODER = "encoder back";

    /*
        Dead wheel directions
     */
    public static final double LEFT_ENCODER_DIRECTION = Encoder.FORWARD;
    public static final double RIGHT_ENCODER_DIRECTION = Encoder.FORWARD;
    public static final double BACK_ENCODER_DIRECTION = Encoder.FORWARD;

    /*
        Arm configuration name
     */
    public static final String ARM_NAME = "arm-servo";

    /*
        Claw configuration name
     */
    public static final String CLAW_NAME = "claw-servo";

    /*
        Lift configuration name
     */
    public static final String LIFT_NAME = "lift-motor";

    /*
        Wrist configuration name
     */
    public static final String WRIST_NAME = "wrist-servo";

    /*
        Pedro's parameters
    */

    // The weight of the robot in Kilograms
    public static final double ROBOT_WEIGHT_IN_KG = 10.5;

    // Maximum velocity of the robot going forward
    public static final double ROBOT_SPEED_FORWARD = 51.4598;

    // Maximum velocity of the robot going right
    public static final double ROBOT_SPEED_LATERAL = 28.7119;

    // Rate of deceleration when power is cut-off when the robot is moving forward
    public static final double FORWARD_ZERO_POWER_ACCEL = -59.805;

    // Rate of deceleration when power is cut-off when the robot is moving to the right
    public static final double LATERAL_ZERO_POWER_ACCEL = -99.672;

    // Determines how fast your robot will decelerate as a factor of how fast your robot will coast to a stop
    public static final double ZERO_POWER_ACCEL_MULT = 3.5;

    /* Centripetal force correction - increase if robot is correcting into the path
                                    - decrease if robot is correcting away from the path */
    public static final double CENTRIPETAL_SCALING = 0.0004;
}
