/* Copyright (c) 2021 FIRST. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided that
 * the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of FIRST nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS
 * LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.firstinspires.ftc.teamcode.cometbots.tests;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.SequentialAction;
import com.acmerobotics.roadrunner.SleepAction;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.eventloop.opmode.LinearOpMode;
import com.qualcomm.robotcore.eventloop.opmode.TeleOp;
import com.qualcomm.robotcore.hardware.Gamepad;
import com.qualcomm.robotcore.util.ElapsedTime;

import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.pedroPathing.localization.Pose;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.BezierLine;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.Path;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.PathChain;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.Point;
import org.firstinspires.ftc.teamcode.subsystem.ArmActionsSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.ClawActionsSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.LiftActionsSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.WristActionsSubsystem;


@TeleOp(name = "SpecimenTest", group = "Debug")
public class SpecimenTest extends LinearOpMode {

    private final ElapsedTime runtime = new ElapsedTime();

    private PathChain path;

    private final Pose startPose = new Pose(36, 72);

    public class MoveToPath implements Action {

        private Follower actionRobot;
        private PathChain pathChain;

        private MoveToPath(PathChain path, Follower robot) {
            this.actionRobot = robot;
            this.pathChain = path;
        }
        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            return false;
        }
    }

    @Override
    public void runOpMode() {


        /*
         * Instantiate Lift
         */
        LiftActionsSubsystem lift = new LiftActionsSubsystem(hardwareMap);
        WristActionsSubsystem wrist = new WristActionsSubsystem(hardwareMap);
        ArmActionsSubsystem arm = new ArmActionsSubsystem(hardwareMap);
        ClawActionsSubsystem claw = new ClawActionsSubsystem(hardwareMap);
        Follower robot = new Follower(hardwareMap);

        /*
          Robot stuff
         */
        robot.setStartingPose(startPose);
        path = robot.pathBuilder() .addPath(
                        // Line 1
                        new BezierLine(
                                new Point(36.000, 72.000, Point.CARTESIAN),
                                new Point(37.500, 72.000, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(0)).build();

        /*
         * Instantiate gamepad state holders
         */
        Gamepad currentGamepad1 = new Gamepad();
        Gamepad previousGamepad1 = new Gamepad();
        Gamepad currentGamepad2 = new Gamepad();
        Gamepad previousGamepad2 = new Gamepad();

        lift.init();
        wrist.init();
        arm.init();
        claw.init();
        robot.setMaxPower(.75);
        robot.followPath(path);

        waitForStart();
        runtime.reset();


        // run until the end of the match (driver presses STOP)
        while (opModeIsActive()) {

            previousGamepad1.copy(currentGamepad1);
            currentGamepad1.copy(gamepad1);

            previousGamepad2.copy(currentGamepad2);
            currentGamepad2.copy(gamepad2);


            robot.setTeleOpMovementVectors(-currentGamepad1.left_stick_y, -currentGamepad1.left_stick_x, -currentGamepad1.right_stick_x);
            //robot.update();

            if (currentGamepad1.dpad_up && !previousGamepad1.dpad_up) {
                arm.setPosition(arm.getPosition() + .05);
            }

            if (currentGamepad1.dpad_down && !previousGamepad1.dpad_down) {
                arm.setPosition(arm.getPosition() - .05);
            }
            if (currentGamepad1.circle && !previousGamepad1.circle) {
                claw.switchState();
            }
            if (currentGamepad1.dpad_right && !previousGamepad1.dpad_right) {
                Actions.runBlocking(
                        new SequentialAction(
                                wrist.toFloorPosition(),
                                lift.toHighRung(),
                                wrist.toSpeciemenBar(),
                                lift.dropToHighRung()

                        )
                );

            }
            if (currentGamepad1.dpad_left && !previousGamepad1.dpad_left) {
                Actions.runBlocking(
                        new SequentialAction(
                                claw.openClaw(),
                                wrist.toFloorPosition(),
                                lift.toFloorPosition()
                        )
                );

            }


            if (currentGamepad1.triangle && !previousGamepad1.triangle) {
                wrist.setPosition(wrist.getPosition() + .05);
            }

            if (currentGamepad1.cross && !previousGamepad1.cross) {
                wrist.setPosition(wrist.getPosition() - .05);
            }

            if (currentGamepad1.right_bumper && !previousGamepad1.right_bumper) {
                lift.setPosition(lift.getPosition() + 175);
            }

            if (currentGamepad1.left_bumper && !previousGamepad1.left_bumper) {
                lift.setPosition(lift.getPosition() - 25);
            }

            if (currentGamepad2.x && !previousGamepad2.x){
                while(true) {
                    robot.update();
                    if (!robot.isBusy()) {
                        break;
                    }
                }
            }


            // Show the elapsed game time and wheel power.
            telemetry.addData("Status", "Run Time: " + runtime.toString());
            telemetry.addData("Lift Drive Position", lift.getPosition());
            telemetry.addData("Wrist Position", wrist.getPosition());
            telemetry.addData("Arm Position", arm.getPosition());

            telemetry.update();
        }
    }
}
