package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.PedroConstants.ARM_NAME;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armFloor;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armPark;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armSubmarine;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

public class ArmActionsSubsystem {

    public enum ArmState {
        PARK, FLOOR, BUCKET, SUBMARINE
    }

    private ServoImplEx arm;
    private ArmState state;

    public ArmActionsSubsystem(HardwareMap hardwareMap) {
        this.arm = hardwareMap.get(ServoImplEx.class, ARM_NAME);
    }

    public class MoveToPosition implements Action {
        private double positionValue;
        private ArmState positionState;

        public MoveToPosition(double positionValue, ArmState positionState) {
            this.positionValue = positionValue;
            this.positionState = positionState;
        }

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            arm.setPosition(positionValue);
            setState(positionState);
            telemetryPacket.put("Arm State", positionState);
            return false;
        }
    }

    public Action toParkPosition() {
        return new MoveToPosition(armPark, ArmState.PARK);
    }

    public Action toSubmarinePosition() {
        return new MoveToPosition(armSubmarine, ArmState.SUBMARINE);
    }

    public Action toFloorPosition() {
        return new MoveToPosition(armFloor, ArmState.FLOOR);
    }

    public Action toBucketPosition() {
        return new MoveToPosition(armBucket, ArmState.BUCKET);
    }

    public void setState(ArmState armState) {
        this.state = armState;
    }

    public ArmState getState() {
        return this.state;
    }

    public void init() {
        this.arm.resetDeviceConfigurationForOpMode();
        Actions.runBlocking(this.toParkPosition());
    }

    public void start() {
        Actions.runBlocking(this.toParkPosition());
    }

    public double getPosition() {
        return this.arm.getPosition();
    }

    public void setPosition(double position) {
        this.arm.setPosition(position);
    }

}
