package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.PedroConstants.CLAW_NAME;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.clawClose;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.clawOpen;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.Servo;

public class ClawActionsSubsystem {

    public enum ClawState {
        CLOSED, OPEN
    }

    private Servo claw;
    private ClawState state;

    public ClawActionsSubsystem(HardwareMap hardwareMap) {
        this.claw = hardwareMap.get(Servo.class, CLAW_NAME);
    }

    public class MoveToPosition implements Action {
        private double positionValue;
        private ClawState positionState;

        public MoveToPosition(double positionValue, ClawState positionState) {
            this.positionValue = positionValue;
            this.positionState = positionState;
        }

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            setState(positionState);
            claw.setPosition(positionValue);
            telemetryPacket.put("Claw State", positionState);
            return false;
        }
    }

    public Action openClaw() {
        return new MoveToPosition(clawOpen, ClawState.OPEN);
    }

    public Action closeClaw() {
        return new MoveToPosition(clawClose, ClawState.CLOSED);
    }

    public void setState(ClawState clawState) {
        this.state = clawState;
    }

    public ClawState getState() {
        return this.state;
    }

    public void switchState() {
        if (state == ClawState.CLOSED) {
            Actions.runBlocking(openClaw());
        } else if (state == ClawState.OPEN) {
            Actions.runBlocking(closeClaw());
        }
    }

    public void init() {
        Actions.runBlocking(openClaw());
    }

    public void start() {
        Actions.runBlocking(openClaw());
    }

    public double getPosition() {
        return this.claw.getPosition();
    }

}